# Generated by Django 3.2.11 on 2022-01-26 18:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('canvas', '0013_auto_20211221_1535'),
    ]

    operations = [
        migrations.AlterField(
            model_name='canvasenterprisecustomerconfiguration',
            name='canvas_base_url',
            field=models.CharField(blank=True, default='', help_text='The base URL used for API requests to Canvas, i.e. https://instructure.com.', max_length=255, verbose_name='Canvas Base URL'),
        ),
        migrations.AlterField(
            model_name='canvasenterprisecustomerconfiguration',
            name='catalogs_to_transmit',
            field=models.TextField(blank=True, default='', help_text='A comma-separated list of catalog UUIDs to transmit. If blank, all customer catalogs will be transmitted. If there are overlapping courses in the customer catalogs, the overlapping course metadata will be selected from the newest catalog.'),
        ),
        migrations.AlterField(
            model_name='canvasenterprisecustomerconfiguration',
            name='channel_worker_username',
            field=models.CharField(blank=True, default='', help_text='Enterprise channel worker username to get JWT tokens for authenticating LMS APIs.', max_length=255),
        ),
        migrations.AlterField(
            model_name='canvasenterprisecustomerconfiguration',
            name='client_id',
            field=models.CharField(blank=True, default='', help_text='The API Client ID provided to edX by the enterprise customer to be used to make API calls to Canvas on behalf of the customer.', max_length=255, verbose_name='API Client ID'),
        ),
        migrations.AlterField(
            model_name='canvasenterprisecustomerconfiguration',
            name='client_secret',
            field=models.CharField(blank=True, default='', help_text='The API Client Secret provided to edX by the enterprise customer to be used to make  API calls to Canvas on behalf of the customer.', max_length=255, verbose_name='API Client Secret'),
        ),
        migrations.AlterField(
            model_name='canvasenterprisecustomerconfiguration',
            name='idp_id',
            field=models.CharField(blank=True, default='', help_text='If provided, will be used as IDP slug to locate remote id for learners', max_length=255),
        ),
        migrations.AlterField(
            model_name='historicalcanvasenterprisecustomerconfiguration',
            name='canvas_base_url',
            field=models.CharField(blank=True, default='', help_text='The base URL used for API requests to Canvas, i.e. https://instructure.com.', max_length=255, verbose_name='Canvas Base URL'),
        ),
        migrations.AlterField(
            model_name='historicalcanvasenterprisecustomerconfiguration',
            name='catalogs_to_transmit',
            field=models.TextField(blank=True, default='', help_text='A comma-separated list of catalog UUIDs to transmit. If blank, all customer catalogs will be transmitted. If there are overlapping courses in the customer catalogs, the overlapping course metadata will be selected from the newest catalog.'),
        ),
        migrations.AlterField(
            model_name='historicalcanvasenterprisecustomerconfiguration',
            name='channel_worker_username',
            field=models.CharField(blank=True, default='', help_text='Enterprise channel worker username to get JWT tokens for authenticating LMS APIs.', max_length=255),
        ),
        migrations.AlterField(
            model_name='historicalcanvasenterprisecustomerconfiguration',
            name='client_id',
            field=models.CharField(blank=True, default='', help_text='The API Client ID provided to edX by the enterprise customer to be used to make API calls to Canvas on behalf of the customer.', max_length=255, verbose_name='API Client ID'),
        ),
        migrations.AlterField(
            model_name='historicalcanvasenterprisecustomerconfiguration',
            name='client_secret',
            field=models.CharField(blank=True, default='', help_text='The API Client Secret provided to edX by the enterprise customer to be used to make  API calls to Canvas on behalf of the customer.', max_length=255, verbose_name='API Client Secret'),
        ),
        migrations.AlterField(
            model_name='historicalcanvasenterprisecustomerconfiguration',
            name='idp_id',
            field=models.CharField(blank=True, default='', help_text='If provided, will be used as IDP slug to locate remote id for learners', max_length=255),
        ),
    ]
