# Generated by Django 3.2.12 on 2022-02-22 17:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0152_add_should_inactivate_other_customers'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisecustomer',
            name='enable_browse_and_request',
            field=models.BooleanField(default=False, help_text='Specifies whether browse and request is enabled for this customer. Managed via admin portal settings UI.'),
        ),
        migrations.AddField(
            model_name='historicalenterprisecustomer',
            name='enable_browse_and_request',
            field=models.BooleanField(default=False, help_text='Specifies whether browse and request is enabled for this customer. Managed via admin portal settings UI.'),
        ),
    ]
