# Generated by Django 3.2.12 on 2022-03-29 13:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('integrated_channel', '0010_genericenterprisecustomerpluginconfiguration'),
    ]

    operations = [
        migrations.AddField(
            model_name='contentmetadataitemtransmission',
            name='plugin_configuration_id',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
    ]
